#include <sqlite3.h>
#include <stdio.h>
#include <string>
#include <iostream>
#include <stdlib.h>
#include <time.h>

#define DATABASE "fortunes.db"
#define GET_QUERY "SELECT fortune FROM fortunes WHERE id == "
#define GET_LENGTH_QUERY "SELECT length FROM fortunes WHERE id == "
#define LONG_SHORT 160
#define NUM_OF_RECORDS 15598

using namespace std;

static int printCallback(void *NotUsed, int argc, char **argv, char **azColName);
static int getLenCallback(void *NotUsed, int argc, char **argv, char **azColName);
void printHelp(char *pathInChars);
void printFortune(bool normalMsg, bool longMsg, int chars, bool wait);
void checkArgSeqAndPrint(int argNum, char *args[]);

int main(int argc, char *argv[]) {
    if (argc > 5) {
        cout << "Too much args given. I'm confused!" << endl;
//        cin.get();
        exit(1);
//    } else if (argc < 2) {
//        cout << "Too few args given. I'm confused!" << endl;
//        cin.get();
        exit(1);
    } else {
        checkArgSeqAndPrint(argc, argv);
    }

    return 0;
}

void printHelp(char *pathInChars) {
    // grab the program name from a path
    string path = pathInChars;
    int pos = path.rfind("\\");
    string name = path.substr(pos+1);

    // now proceed to print the help
    cout << name << " [-h] [-l [-n]] [-s [-n]] [-w]" << endl;
    cout << "   -h  -- Print this help screen" << endl;
    cout << "   -l  -- Use only quotations longer than the length specified with -n,\n\tor 160 characters if -n is not used" << endl;
    cout << "   -s  -- Use only quotations shorter than the length specified with -n,\n\tor 160 characters if -n is not used" << endl;
    cout << "   -n  -- Override the length used by -l and -s to determine\n\t\"long\" and \"short\" messages (default 160 characters)" << endl;
    cout << "   -w  -- Wait for a user input before terminating" << endl;
}

void printFortune(bool normalMsg, bool longMsg, int chars, bool wait) {
    sqlite3 *dbObj;
    sqlite3_open(DATABASE, &dbObj);
    if (normalMsg) {
        srand(time(NULL));
        int randFortune = rand() % NUM_OF_RECORDS + 1;
        char randFortuneChars[5];
        itoa(randFortune, randFortuneChars, 10);
        string getQString = GET_QUERY;
        getQString.append(randFortuneChars);
        char *getQChars = (char*)getQString.c_str();
        char *errorMsg;
        long ok = sqlite3_exec(dbObj, getQChars, printCallback, 0, &errorMsg);
    } else {
        bool reqMet = false;
        while (!reqMet) {
            srand(time(NULL));
            int randFortune = rand() % chars + 1;
            char randFortuneChars[5];
            itoa(randFortune, randFortuneChars, 10);
            string getLenQString = GET_LENGTH_QUERY;
            getLenQString.append(randFortuneChars);
            char *getLenQ = (char*)getLenQString.c_str();
            char *errorMsg;
            int len;
            long ok = sqlite3_exec(dbObj, getLenQ, getLenCallback, (void*)&len, &errorMsg);
            if (ok == SQLITE_OK) {
                if ((longMsg && (len >= chars)) || (!longMsg && (len <= chars))) {
                    reqMet = true;
                    string getQString = GET_QUERY;
                    getQString.append(randFortuneChars);
                    char *getQChars = (char*)getQString.c_str();
                    sqlite3_exec(dbObj, getQChars, printCallback, 0, &errorMsg);
                }
            } else {
                cout << "Error: " << errorMsg << endl;
                sqlite3_free(errorMsg);
                sqlite3_free(dbObj);
                if (wait) {
                    cin.get();
                }
                exit(1);
            }
        }
    }
    if (wait) {
        cin.get();
    }
}

void checkArgSeqAndPrint(int argNum, char *args[]) {
    // check if correct sequence of args was given
    int i = 1;
    bool hasLong = false;
    bool hasShort = false;
    bool shouldWait = false;
    bool normalMsg = true;
    bool printHelpBool = false;
    int chars = LONG_SHORT;
    for (i; i < argNum; i++) {
        if (strcmp(args[i], "-l") == 0) {
            hasLong = true;
            normalMsg = false;
        } else if (strcmp(args[i], "-s") == 0) {
            hasShort = true;
            normalMsg = false;
        } else if (strcmp(args[i], "-n") == 0) {
            if (isdigit(*args[i+1])) {
                chars = atoi(args[i+1]);
            }
        } else if (strcmp(args[i], "-w") == 0) {
            shouldWait = true;
        } else if (strcmp(args[i], "-h") == 0) {
            printHelpBool = true;
        }
    }
    if (hasLong && hasShort) {
        cout << "Wrong sequence of args given. I'm confused!" << endl;
        normalMsg = true;
//        cin.get();
    } else if (printHelpBool) {
        printHelp(args[0]);
    } else {
        printFortune(normalMsg, hasLong, chars, shouldWait);
    }
}

// callback that prints stuff when data is returned from a database
static int printCallback(void *NotUsed, int argc, char **argv, char **azColName) {
    cout << argv[0] << endl;
    return 0;
}
// callback that returns stuff when data is returned from a database
static int getLenCallback(void *NotUsed, int argc, char **argv, char **azColName) {
    //cout << argv[0] << endl;
    *(int*)NotUsed = atoi(argv[0]);
    return 0;
}
